@@ -1,7 +1,8 @@
 /**
  * @file irc-more.c A couple of additional IRC features.
  *
- * Copyright (C) 2007 Sadrul Habib Chowdhury <sadrul@users.sourceforge.net>
+ * Copyright (C) 2007-2008 Sadrul Habib Chowdhury <sadrul@users.sourceforge.net>
+ * Copyright (C) 2007-2008 John Bailey <rekkanoryo@rekkanoryo.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +19,7 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02111-1301  USA
  */
 
-/* Pack/Local headers */
+/* If you can't figure out what this line is for, DON'T TOUCH IT. */
 #include "../common/pp_internal.h"
 
 #include <accountopt.h>
@@ -50,13 +51,18 @@ show_them(gpointer data)
 {
 	/* So you think you can kick me? I'll show you! */
 	PurpleConversation *conv = data;
-	char *command = g_strdup_printf("join %s", purple_conversation_get_name(conv));
-	char *markup = g_markup_escape_text(command, -1);
-	char *error = NULL;
-	purple_cmd_do_command(conv, command, markup, &error);  /* Do anything with the return value? */
-	g_free(command);
-	g_free(markup);
-	g_free(error);
+	char *conv_name = NULL, *command = NULL, *markup = NULL, *error = NULL;
+
+	if(conv_name) {
+		command = g_strdup_printf("join %s", conv_name);
+		markup = g_markup_escape_text(command, -1);
+		error = NULL;
+		purple_cmd_do_command(conv, command, markup, &error);  /* Do anything with the return value? */
+		g_free(command);
+		g_free(markup);
+		g_free(error);
+	}
+
 	return FALSE;
 }
 
@@ -122,6 +128,7 @@ signed_on_cb(PurpleConnection *gc)
 	return;
 }
 
+#if !PURPLE_VERSION_CHECK(2,4,0)
 static PurpleCmdRet
 notice_cmd_cb(PurpleConversation *conv, const gchar *cmd, gchar **args,
 		gchar **error, void *data)
@@ -164,19 +171,20 @@ notice_cmd_cb(PurpleConversation *conv, const gchar *cmd, gchar **args,
 	irc_info->send_raw(gc, msg, len);
 
 	/* avoid a possible double-free crash */
-	if(msg != tmp);
+	if(msg != tmp)
 		g_free(tmp);
 
 	g_free(msg);
 
 	return PURPLE_CMD_RET_OK;
 }
+#endif
 
 static void
 irc_sending_text(PurpleConnection *gc, char **msg, gpointer null)
 {
 	PurpleAccount *account = purple_connection_get_account(gc);
-	char **old = msg;
+	char *old = *msg;
 
 	if (MATCHES("QUIT ")) {
 		char *message = strchr(*msg, ':');
@@ -197,8 +205,8 @@ irc_sending_text(PurpleConnection *gc, char **msg, gpointer null)
 		*version = '\0';
 		*msg = g_strdup_printf("%s:\001VERSION %s\001\r\n", *msg, CTCP_REPLY);
 	}
-	if (msg != old)
-		g_free(*old);
+	if (*msg != old)
+		g_free(old);
 }
 
 static gboolean
@@ -218,9 +226,11 @@ plugin_load(PurplePlugin *plugin)
 	/* specify our help string and register our command */
 	notice_help = _("notice target message:  Send a notice to the specified target.");
 
+#if !PURPLE_VERSION_CHECK(2,4,0)
 	notice_cmd_id = purple_cmd_register("notice", "ws", PURPLE_CMD_P_PLUGIN,
 			PURPLE_CMD_FLAG_IM | PURPLE_CMD_FLAG_CHAT | PURPLE_CMD_FLAG_PRPL_ONLY,
 			"prpl-irc", notice_cmd_cb, notice_help, NULL);
+#endif
 
 	/* we need this handle for the signed-on signal */
 	gc_handle = purple_connections_get_handle();
