@@ -1,7 +1,7 @@
 /*
  * IRC Helper Plugin for libpurple
  *
- * Copyright (C) 2005-2007, Richard Laager <rlaager@pidgin.im>
+ * Copyright (C) 2005-2008, Richard Laager <rlaager@pidgin.im>
  * Copyright (C) 2004-2005, Mathias Hasselmann <mathias@taschenorakel.de>
  * Copyright (C) 2005, Daniel Beardsmore <uilleann@users.sf.net>
  * Copyright (C) 2005, Björn Nilsson <BNI on irc.freenode.net>
@@ -23,6 +23,7 @@
  * 02111-1301, USA.
  */
 
+/* If you can't figure out what this line is for, DON'T TOUCH IT. */
 #include "../common/pp_internal.h"
 
 #include <string.h>
@@ -55,6 +56,7 @@
 #define AUTO_RESPONSE_INTERVAL 600
 
 #define DOMAIN_SUFFIX_FREENODE ".freenode.net"
+#define DOMAIN_SUFFIX_FUNCOM ".funcom.com"
 #define DOMAIN_SUFFIX_GAMESURGE ".gamesurge.net"
 #define DOMAIN_SUFFIX_THUNDERCITY ".thundercity.org"
 #define DOMAIN_SUFFIX_DALNET ".dal.net"
@@ -84,6 +86,7 @@
 #define MESSAGE_NICKSERV_CLOAKED " set your hostname to"
 #define MESSAGE_NICKSERV_ID_FAILURE "Password Incorrect"
 #define MESSAGE_NICKSERV_IDENTIFIED "Password accepted - you are now recognized"
+#define MESSAGE_SPOOFING_YOUR_IP "*** Spoofing your IP. congrats."
 #define MESSAGE_QUAKENET_Q_CRUFT \
 	"Remember: NO-ONE from QuakeNet will ever ask for your password.  " \
 	"NEVER send your password to ANYONE except Q@CServe.quakenet.org."
@@ -103,6 +106,7 @@
 
 #define NICK_CHANSERV "ChanServ"
 #define NICK_FREENODE_CONNECT "freenode-connect"
+#define NICK_FUNCOM_Q_SERVICE NICK_QUAKENET_Q "@cserve.funcom.com"
 #define NICK_GAMESURGE_AUTHSERV "AuthServ"
 #define NICK_GAMESURGE_AUTHSERV_SERVICE NICK_GAMESURGE_AUTHSERV "@Services.GameSurge.net"
 #define NICK_GAMESURGE_GLOBAL "Global"
@@ -138,6 +142,7 @@ typedef enum {
 	IRC_NETWORK_TYPE_UNDERNET  = 0x0200,
 	IRC_NETWORK_TYPE_THUNDERCITY = 0x0400,
 	IRC_NETWORK_TYPE_DALNET    = 0x0800,
+	IRC_NETWORK_TYPE_FUNCOM    = 0x1000,
 } IRCHelperStateFlags;
 
 struct proto_stuff
@@ -198,7 +203,12 @@ static PurpleConversation *get_conversation(PurpleAccount *account)
 
 	conv = g_new0(PurpleConversation, 1);
 	conv->type = PURPLE_CONV_TYPE_IM;
-	purple_conversation_set_account(conv, account);
+	/* If we use this then the conversation updated signal is fired and
+	 * other plugins might start doing things to our conversation, such as
+	 * setting data on it which we would then need to free etc. It's easier
+	 * just to be more hacky by setting account directly. */
+	/* purple_conversation_set_account(conv, account); */
+	conv->account = account;
 
 	return conv;
 }
@@ -227,6 +237,8 @@ static IRCHelperStateFlags get_connection_type(PurpleConnection *connection)
 		type = IRC_NETWORK_TYPE_DALNET;
 	else if (g_str_has_suffix(username, DOMAIN_SUFFIX_QUAKENET))
 		type = IRC_NETWORK_TYPE_QUAKENET;
+	else if (g_str_has_suffix(username, DOMAIN_SUFFIX_FUNCOM))
+		type = IRC_NETWORK_TYPE_FUNCOM;
 	else if (g_str_has_suffix(username, DOMAIN_SUFFIX_JEUX))
 		type = IRC_NETWORK_TYPE_JEUX;
 	else if (g_str_has_suffix(username, DOMAIN_SUFFIX_UNDERNET))
@@ -282,7 +294,7 @@ static void authserv_identify(const char *command, PurpleConnection *connection,
 		const gchar *authserv = NICK_AUTHSERV;
 		gchar *authentication = g_strconcat(command, " ", username, " ", password, NULL);
 
-		purple_debug_misc(PLUGIN_STATIC_NAME, "Sending authentication: %s\n", authentication);
+		purple_debug_misc(PLUGIN_STATIC_NAME, "Sending authentication: %s %s <PASSWORD>\n", command, username);
 
 		g_hash_table_insert(states,
 		                    connection->proto_data,
@@ -294,11 +306,15 @@ static void authserv_identify(const char *command, PurpleConnection *connection,
 			authserv = NICK_DALNET_AUTHSERV_SERVICE;
 		else if (state & IRC_NETWORK_TYPE_QUAKENET)
 			authserv = NICK_QUAKENET_Q_SERVICE;
+		else if (state & IRC_NETWORK_TYPE_FUNCOM)
+			authserv = NICK_FUNCOM_Q_SERVICE;
 		else if (state & IRC_NETWORK_TYPE_UNDERNET)
 			authserv = NICK_UNDERNET_X;
 
 		serv_send_im(connection, authserv, authentication, 0);
 
+		g_free(authentication);
+
 		/* Register a timeout... If we don't get the expected response from AuthServ,
 		 * we need to stop suppressing messages from it at some point or the user
 		 * could be very confused.
@@ -337,7 +353,7 @@ static void jeux_identify(PurpleConnection *connection, IRCHelperStateFlags stat
 		PurpleConversation *conv = get_conversation(account);
 		gchar *error;
 
-		purple_debug_misc(PLUGIN_STATIC_NAME, "Sending authentication: %s\n", authentication);
+		purple_debug_misc(PLUGIN_STATIC_NAME, "Sending authentication: quote %s login %s <PASSWORD>\n", NICK_JEUX_Z, username);
 
 		g_hash_table_insert(states,
 		                    connection->proto_data,
@@ -349,6 +365,7 @@ static void jeux_identify(PurpleConnection *connection, IRCHelperStateFlags stat
 			g_free(error);
 		}
 		g_free(conv);
+		g_free(authentication);
 
 		/* Register a timeout... If we don't get the expected response from AuthServ,
 		 * we need to stop suppressing messages from it at some point or the user
@@ -400,8 +417,13 @@ static void nickserv_do_identify(char *authentication, gpointer proto_data, Purp
 {
 	PurpleConversation *conv = get_conversation(purple_connection_get_account(gc));
 	gchar *error;
+	gchar *tmp;
 
-	purple_debug_misc(PLUGIN_STATIC_NAME, "Sending authentication: %s\n", authentication);
+	purple_debug_misc(PLUGIN_STATIC_NAME, "Sending authentication: %s <PASSWORD>\n", authentication);
+
+	tmp = g_strconcat(authentication, " ", nickpassword, NULL);
+	g_free(authentication);
+	authentication = tmp;
 
 	if (purple_cmd_do_command(conv, authentication, authentication, &error) != PURPLE_CMD_STATUS_OK)
 	{
@@ -421,13 +443,13 @@ static void nickserv_do_identify(char *authentication, gpointer proto_data, Purp
 
 static void nickserv_identify(gpointer proto_data, PurpleConnection *gc, const char *nickpassword)
 {
-	char *authentication = g_strdup_printf("quote %s IDENTIFY %s", NICK_NICKSERV, nickpassword);
+	char *authentication = g_strdup_printf("quote %s IDENTIFY", NICK_NICKSERV);
 	nickserv_do_identify(authentication, proto_data, gc, nickpassword);
 }
 
 static void nickserv_msg_identify(const char *command, gpointer proto_data, PurpleConnection *gc, const char *nickpassword)
 {
-	char *authentication = g_strdup_printf("quote PRIVMSG %s : %s %s", NICK_NICKSERV, command, nickpassword);
+	char *authentication = g_strdup_printf("quote PRIVMSG %s : %s", NICK_NICKSERV, command);
 	nickserv_do_identify(authentication, proto_data, gc, nickpassword);
 }
 
@@ -546,6 +568,13 @@ static void signed_on_cb(PurpleConnection *connection)
 
 		authserv_identify("login ", connection, state);
 	}
+	else if (state & IRC_NETWORK_TYPE_FUNCOM)
+	{
+		purple_debug_info(PLUGIN_STATIC_NAME, "Connected with Funcom: %s\n",
+		                purple_connection_get_display_name(connection));
+
+		authserv_identify("AUTH", connection, state);
+	}
 	else
 	{
 		nickpassword = purple_account_get_string(account, PLUGIN_ID "_nickpassword", "");
@@ -573,7 +602,7 @@ static void signed_on_cb(PurpleConnection *connection)
 				command = g_strdup_printf("quote %s GHOST %s %s", NICK_NICKSERV, userparts[0], nickpassword);
 				conv = get_conversation(account);
 
-				purple_debug_misc(PLUGIN_STATIC_NAME, "Sending command: %s\n", command);
+				purple_debug_misc(PLUGIN_STATIC_NAME, "Sending command: quote %s GHOST %s <PASSWORD>\n", NICK_NICKSERV, userparts[0]);
 
 				if (purple_cmd_do_command(conv, command, command, &error) != PURPLE_CMD_STATUS_OK)
 				{
@@ -885,6 +914,12 @@ static gboolean receiving_im_msg_cb(PurpleAccount *account, gchar **sender, gcha
 		return TRUE;
 	}
 
+	/* Suppress "IP spoofing" notice on worldforge.org:
+	 *   http://plugins.guifications.org/trac/ticket/524 */
+	if (g_str_has_prefix(msg, MESSAGE_PURPLE_NOTICE_PREFIX MESSAGE_SPOOFING_YOUR_IP))
+	{
+		return TRUE;
+	}
 
 	/* SLIGHTLY COMPLICATED SUPPRESSION RULES */
 
@@ -1114,7 +1149,8 @@ static gboolean receiving_im_msg_cb(PurpleAccount *account, gchar **sender, gcha
 	}
 
 	/* Suppress useless Q notifications if we're identifying automatically. */
-	if (state & IRC_NETWORK_TYPE_QUAKENET &&
+	if ((state & IRC_NETWORK_TYPE_QUAKENET ||
+	     state & IRC_NETWORK_TYPE_FUNCOM) &&
 	    state & IRC_WILL_ID &&
 	    g_str_equal(nick, NICK_QUAKENET_Q))
 	{
