@@ -1,10 +1,10 @@
 /*
  * slashexec - A CLI for libpurple clients
- * Copyright (C) 2004-2007 Gary Kramlich
- * Copyright (C) 2005-2007 Peter Lawler
- * Copyright (C) 2005-2007 Daniel Atallah
- * Copyright (C) 2005-2007 John Bailey
- * Copyright (C) 2006-2007 Sadrul Habib Chowdhury
+ * Copyright (C) 2004-2008 Gary Kramlich
+ * Copyright (C) 2005-2008 Peter Lawler
+ * Copyright (C) 2005-2008 Daniel Atallah
+ * Copyright (C) 2005-2008 John Bailey
+ * Copyright (C) 2006-2008 Sadrul Habib Chowdhury
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -26,6 +26,7 @@
  * care to translate messages users will see in the main interface.  Debug
  * window messages are not important. - rekkanoryo */
 
+/* If you can't figure out what this line is for, DON'T TOUCH IT. */
 #include "../common/pp_internal.h"
 
 #include <stdarg.h>
@@ -95,6 +96,25 @@ se_replace_ending_char(gchar *string, gchar replace)
 	return;
 }
 
+static gchar *
+se_strdelimit(gchar *string, gchar newdelim)
+{ /* This function borrowed and tweaked from glib to suit my purposes */
+
+	/* these are the decimal representations of the ascii control characters that
+	 * we need to remove to prevent bad behavior, such as XMPP disconnects */
+	gchar delimiters[] = { 1, 2, 3, 4, 5, 6, 7, 8, 11, 12, 14, 15, 16, 17,
+						18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31 };
+	gchar *c;
+
+	g_return_val_if_fail(string !=NULL, NULL);
+
+	for(c = string; *c; c = g_utf8_next_char(c))
+		if(strchr(delimiters, *c))
+			*c = newdelim;
+
+	return string;
+}
+
 static gboolean
 se_do_action(PurpleConversation *conv, gchar *args, gboolean send)
 {
@@ -243,6 +263,18 @@ se_do_action(PurpleConversation *conv, gchar *args, gboolean send)
 		if(send) {
 			purple_debug_info("slashexec", "Command stdout: %s\n", cmd_stdout);
 
+			if(!g_utf8_validate(cmd_stdout, -1, NULL)) {
+				purple_debug_error("slashexec", "Output failed UTF-8 verification!\n");
+
+				return FALSE;
+			} else {
+				cmd_stdout = se_strdelimit(cmd_stdout, ' ');
+
+				g_strstrip(cmd_stdout);
+
+				purple_debug_info("slashexec", "Sanitized command stdout: %s\n", cmd_stdout);
+			}
+
 			switch(purple_conversation_get_type(conv)) {
 				case PURPLE_CONV_TYPE_IM:
 					purple_conv_im_send(PURPLE_CONV_IM(conv), cmd_stdout);
@@ -354,6 +386,9 @@ static void
 se_sending_chat_msg_cb(PurpleAccount *account, char **message, int id)
 {
 	PurpleConversation *conv;
+	if (message == NULL || *message == NULL)  /* It's possible if some other callback to
+	                                           to the signal resets the message */
+		return;
 	conv = purple_find_chat(account->gc, id);
 	se_sending_msg_helper(conv, message);
 }
@@ -429,11 +464,11 @@ get_plugin_pref_frame(PurplePlugin *plugin)
 	purple_plugin_pref_frame_add(frame, pref);
 
 	pref = purple_plugin_pref_new_with_name_and_label(PREF_SLASH,
-					_("/exec Command (/exec some_command)"));
+					_("/exec Command (/exec someCommand)"));
 	purple_plugin_pref_frame_add(frame, pref);
 
 	pref = purple_plugin_pref_new_with_name_and_label(PREF_BANG,
-					_("Exclamation point (!some_command)"));
+					_("Exclamation point (!someCommand)"));
 	purple_plugin_pref_frame_add(frame, pref);
 
 	return frame;
